import docker
from prefect import task


@task
def submit_docker_job(job_definition: dict, path: str) -> str:
    client = docker.APIClient(base_url="unix://var/run/docker.sock")
    host_config = client.create_host_config(binds=[path + ":/mnt"])

    container = client.create_container(**job_definition, host_config=host_config)
    container_id = container.get("Id")

    client.start(container=container_id)

    return container_id



from .make_docker_job import make_docker_job
from .submit_docker_job import submit_docker_job
from .check_docker_job import check_docker_job
from .terminate_docker_job import terminate_docker_job
from .get_docker_logs import get_docker_logs
from .clean_docker_job import clean_docker_job



import docker
from prefect import task


@task
def get_docker_logs(container_id: str, log_filter: str) -> str:
    client = docker.APIClient(base_url="unix://var/run/docker.sock")
    logs = client.logs(container=container_id).decode("utf-8")

    log_items = logs.split("\n")
    if "-" in log_filter:
        log_filter = log_filter.replace("-", "")
        log_items = [item for item in log_items if log_filter not in item]
    else:
        log_items = [item for item in log_items if log_filter in item]

    return "\n".join(log_items)



import docker
from prefect import task


@task
def clean_docker_job(container_id: str) -> None:
    client = docker.APIClient(base_url="unix://var/run/docker.sock")
    status = client.containers(all=True, filters={"id": container_id})[0]["State"]

    if status != "running":
        client.remove_container(container=container_id)



from typing import Union

import docker

import prefect
from prefect import task
from prefect.orion.schemas.states import State, Failed

RETRIES_EXCEEDED_EXIT_CODE = 80


@task
def check_docker_job(container_id: str, max_retries: int) -> Union[int, State]:
    task_run = prefect.context.get_run_context().task_run  # type: ignore

    if task_run.run_count > max_retries:
        return RETRIES_EXCEEDED_EXIT_CODE

    client = docker.APIClient(base_url="unix://var/run/docker.sock")
    status = client.containers(all=True, filters={"id": container_id})[0]["State"]

    if status == "running":
        return Failed()

    exitcode = client.wait(container_id, timeout=1)["StatusCode"]
    return exitcode



from prefect import task
import pandas as pd


@task
def find_missing_conditions(conditions, seeds, extensions, manifest: pd.DataFrame):
    missing_conditions = []

    for condition in conditions:
        all_keys = {f"{condition['key']}_{seed:04}": seed for seed in seeds}
        manifest_keys = manifest[manifest["KEY"].isin(all_keys.keys())]

        missing_keys = set(all_keys) - set(manifest_keys["KEY"].unique())
        incomplete_keys = set(
            manifest_keys.groupby("KEY").filter(
                lambda x: len(set(extensions) - set(x["EXTENSION"])) != 0
            )["KEY"]
        )

        for key in list(missing_keys) + list(incomplete_keys):
            missing_condition = condition.copy()
            missing_condition["seed"] = all_keys[key]
            missing_conditions.append(missing_condition)

    return missing_conditions



import docker
from prefect import task


@task
def terminate_docker_job(container_id: str) -> None:
    client = docker.APIClient(base_url="unix://var/run/docker.sock")
    client.stop(container=container_id, timeout=1)



from prefect import task


@task
def make_docker_job(name: str, image: str, index: int) -> dict:
    return {
        "image": image,
        "name": f"{name}_{index}",
        "environment": [
            "SIMULATION_TYPE=LOCAL",
            f"FILE_SET_NAME={name}",
            f"JOB_ARRAY_INDEX={index}",
        ],
        "volumes": ["/mnt"],
    }



from prefect import task
from jinja2 import Template, StrictUndefined


@task
def generate_input_contents(template: str, conditions):
    compiled_template = Template(template, undefined=StrictUndefined)
    rendered_templates = [compiled_template.render(condition) for condition in conditions]
    return rendered_templates
